( cd usr/lib ; rm -rf libgnomecups-1.0.so.1 )
( cd usr/lib ; ln -sf libgnomecups-1.0.so.1.0.0 libgnomecups-1.0.so.1 )
( cd usr/lib ; rm -rf libgnomecups-1.0.so )
( cd usr/lib ; ln -sf libgnomecups-1.0.so.1.0.0 libgnomecups-1.0.so )
